@@ -16,8 +16,7 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef EGG_FRAME_SOURCE_H
-#define EGG_FRAME_SOURCE_H
+#pragma once
 
 #include <glib.h>
 
@@ -27,6 +26,4 @@ guint egg_frame_source_add (guint       frames_per_sec,
                             GSourceFunc callback,
                             gpointer    user_data);
 
-G_END_DECLS
-
-#endif /* EGG_FRAME_SOURCE_H */
+G_END_DECLS
\ No newline at end of file
