@@ -19,16 +19,15 @@
    Author: Darin Adler <darin@bentspoon.com>
 */
 
+#pragma once
+
+#include <eel/eel-vfs-extensions.h>
 #include <gio/gio.h>
-#include <eel/eel-vfs-extensions.h>
+#include <nautilus-extension.h>
+
 #include "nautilus-directory.h"
-#include "nautilus-file-queue.h"
 #include "nautilus-file.h"
-#include "nautilus-monitor.h"
-#include <nautilus-extension.h>
-#include <libxml/tree.h>
 
-typedef struct LinkInfoReadState LinkInfoReadState;
 typedef struct FileMonitors FileMonitors;
 typedef struct DirectoryLoadState DirectoryLoadState;
 typedef struct DirectoryCountState DirectoryCountState;
@@ -41,7 +40,6 @@ typedef struct MountState MountState;
 typedef struct FilesystemInfoState FilesystemInfoState;
 
 typedef enum {
-	REQUEST_LINK_INFO,
 	REQUEST_DEEP_COUNT,
 	REQUEST_DIRECTORY_COUNT,
 	REQUEST_FILE_INFO,
@@ -122,8 +120,6 @@ struct NautilusDirectoryDetails
 	MountState *mount_state;
 
 	FilesystemInfoState *filesystem_info_state;
-	
-	LinkInfoReadState *link_info_read_state;
 
 	GList *file_operations_in_progress; /* list of FileOperation * */
 };
