@@ -28,6 +28,13 @@ typedef enum {
   NAUTILUS_SEARCH_PROVIDER_STATUS_RESTARTING
 } NautilusSearchProviderStatus;
 
+typedef enum {
+  NAUTILUS_SEARCH_ENGINE_ALL_ENGINES,
+  NAUTILUS_SEARCH_ENGINE_TRACKER_ENGINE,
+  NAUTILUS_SEARCH_ENGINE_RECENT_ENGINE,
+  NAUTILUS_SEARCH_ENGINE_MODEL_ENGINE,
+  NAUTILUS_SEARCH_ENGINE_SIMPLE_ENGINE,
+} NautilusSearchEngineTarget;
 
 #define NAUTILUS_TYPE_SEARCH_PROVIDER (nautilus_search_provider_get_type ())
 
